"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * conversion of basic types and objects
 * For basic types: we will have some rules,
 * for more info, could check here: https://github.com/alibaba/pipcook/wiki/%E6%83%B3%E8%A6%81%E4%BD%BF%E7%94%A8python%EF%BC%9F
 *
 */
function conversion(target) {
    if (target.__pipcook__args) {
        let args = '';
        for (let key in target) {
            if (key !== '__pipcook__args') {
                args += `${key}=${conversion(target[key])},`;
            }
        }
        return args.slice(0, args.length - 1);
    }
    else if (typeof target === 'number') {
        return target;
    }
    else if ((typeof target === 'string' || target instanceof String)) {
        return '"' + target + '"';
    }
    else if (typeof target === 'boolean') {
        if (target) {
            return 'True';
        }
        else {
            return 'False';
        }
    }
    else if (target === null || target === undefined) {
        return 'None';
    }
    else if (target.__pipcook__identifier) {
        return target.__pipcook__identifier;
    }
    else if (Array.isArray(target)) {
        let result = '(';
        target.forEach((item) => {
            result += `${conversion(item)},`;
        });
        result = result.slice(0, result.length - 1);
        result += ')';
        return result;
    }
    else {
        return JSON.stringify(target);
    }
}
exports.conversion = conversion;
/**
 * create the proxy object to represent python object.
 * This can change the behavious of get and set and function call of objects so that
 * we can tranlate related codes to python codes.
 * @param object : objected to be proxyed
 * @param statements : current python statements to be executed
 */
function createProxy(object, statements) {
    const p = new Proxy(object, {
        get(target, key) {
            if (key === 'then' || typeof key !== 'string' || key === '__pipcook__identifier'
                || key === 'inspect' || key === 'prototype' || key === 'constructor'
                || key === 'call' || key === 'toString' || key === 'name' || key === '__pipcook__args') {
                return target[key];
            }
            const identifier = 'id_' + Date.now() + parseInt(String(Math.random() * 100000));
            statements.push(`${identifier} = ${target.__pipcook__identifier}.${String(key)}`);
            return getObject(identifier, statements);
        },
        set(target, prop, value) {
            if (typeof prop !== 'string') {
                target[prop] = value;
            }
            else {
                statements.push(`${target.__pipcook__identifier}.${prop.toString()} = ${conversion(value)}`);
            }
            return true;
        },
        apply: function (target, thisArg, argumentsList) {
            const identifier = 'id_' + Date.now() + parseInt(String(Math.random() * 100000));
            const argumentElements = argumentsList.map((ele) => conversion(ele));
            statements.push(`${identifier} = ${target.__pipcook__identifier}(${argumentElements.join(',')})`);
            return getObject(identifier, statements);
        }
    });
    return p;
}
/**
 * get the proxy of Python object from id
 * @param identifier: the identifier used in python codes that linked between python and object
 */
function getObject(identifier, statements) {
    const object = () => { };
    object.__pipcook__identifier = identifier;
    const proxy = createProxy(object, statements);
    return proxy;
}
exports.getObject = getObject;
//# sourceMappingURL=python-helper.js.map